/*
	AUTHOR: YUHONG ZHANG
	FILE NAME: InputParse.cpp
	Function: Parse the inputs of users', which are used by other functions
	Latest updated : 05/01/2011
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <omp.h>
#include <mpi.h>
#include <generic.h>
#include <InputParse.h>

#include <iostream>
using namespace std;
static const char verdate[] = "Version: 1.1, Date: 09/29/2012";
CInputParse::CInputParse()
{	
	iniStr(MatrixFilePath, MAX_PATH_LENGTH);
	iniStr(DBFile, MAX_PATH_LENGTH);
	iniStr(seqQueryFile, MAX_PATH_LENGTH);
	iniStr(seqClsFile, MAX_PATH_LENGTH);
	
	shuffles_ML 	= DEFAULT_SHUFFLE;
	numThreads 		= DEFAULT_THREADS;
	para_type		= DEFAULT_PARA_TYPE;
}

CInputParse::~CInputParse()
{
}

/*
	@-query <string>: specify the query file name. 
	Notice that the query file just contains the query sequence name [whose length is 6],
	not contains the real sequence contex, because query sequences here come from database,
	Providing the names of query sequences is enough.
	If the query sequences do not come from database, you need to revised program to read 
	query sequences.
	
	@-db <string> : specify the database file name.
	which contains all the subject sequences.
	
	@-mat <string> : specified the path of folder
	which contains all the PSSMs of query sequences
	The PSSMs are generated in advance by ncbi blast+(version : 2.2.25+) with the parameters as follows:
		@@evalue: 			10
		@@matrix: 			BLOSUM62
		@@gapopen:			11
		@@gapextend: 		1
		@@num_iteration: 	5
	
	Each query sequence has its PSSM file in this folder.
	So, the number of PSSM file should be the same as the number of query sequences
	
	@-num <integer> : specify the number of shuffling the choosen subject 
		sequence from database. Its default values is 1000.
	
*/
void CInputParse::printUsage()
{
    fprintf(stderr,"\n---------------------------------------------------------------\n");
	fprintf(stderr,"\t%s\n",verdate);
	fprintf(stderr,"\tEstimating pairwise statistical significance using PSSM\n");
	fprintf(stderr,"\tSSM: Standard substitution matrix\n");
	fprintf(stderr,"\tPSSM: Position-specific substitution matrix\n");
	fprintf(stderr,"\n---------------------------------------------------------------\n");	
	fprintf(stderr,"Usage:\n");
    fprintf(stderr,"./Par-PSSE [options]\n\n");
	fprintf(stderr,"\t-query <string>\t: specify sequence queryname file\n");
	fprintf(stderr,"\t-db <string>\t: specify database \n");
	fprintf(stderr,"\t-cls <string>\t: specify classification file \n");
	fprintf(stderr,"\t-mtype <integer>\t: specify the type of matrix\n");
	fprintf(stderr,"\t\t 0: SSM, 1: PSSM\n");
	fprintf(stderr,"\t-mat <string>\t: specify the path for substitution matrix \n");
	fprintf(stderr,"\t\tFor SSM, need to specify the file name\n");
	fprintf(stderr,"\t\tFor PSSM, JUST need specify the path that include PSSM files\n");		
	fprintf(stderr,"\t-num <integer>\t: specify number of shuffles (default: 1000)\n");
	fprintf(stderr,"\t-thr <integer>\t: specify number of threads (default: 1)\n");
	fprintf(stderr,"\t-ptype <integer>: specify parallelism type(default: 0)\n");
	fprintf(stderr,"\t\t\t( 0 : inter-parallelism, 1: intra-parallelism ) \n\n");
	
}
bool CInputParse::parseParams(int argc, char* argv[])
{
	bool DBAvail 		= false;
	//bool pssmPathAvail 	= false;
	bool MatrixPathAvail = false;
	bool QueryAvail 	= false;
	bool ClsAvail 		= false;

	int index;
	int number;
	int pRank = MPI::COMM_WORLD.Get_rank();
	
	//display usages
	if (argc < 2 || !strcmp(argv[1], "-help") || !strcmp(argv[1], "-?")){
		if (pRank == 0){
			printUsage();
		}
		return false;
	}
	index = 1;
	// parse other arguments
	char* arg;
	for ( ; index < argc; index++) {
		arg = argv[index];	
		if (index >= argc){
		if (pRank == 0){
			fprintf(stderr, "The number of the specified argments does not match!");
		}
		return false;
		}				
		if (strcmp(arg, "-query") == 0){
			iniStr(seqQueryFile, MAX_PATH_LENGTH); 
			sscanf(argv[++index], "%s", seqQueryFile);
			QueryAvail = true;
		}else if (strcmp(arg, "-db") == 0){
			iniStr(DBFile, MAX_PATH_LENGTH);
			sscanf(argv[++index], "%s", DBFile);
			DBAvail = true;
		}else if (strcmp(arg, "-cls") == 0){
			iniStr(seqClsFile, MAX_PATH_LENGTH);
			sscanf(argv[++index], "%s", seqClsFile);
			ClsAvail = true;			
		}else if (strcmp(arg, "-mtype") == 0) {
			sscanf(argv[++index], "%d", &number);
			if(number == 0 || number == 1){
				mtype = number;
			} else {
				if (pRank == 0){
					fprintf(stderr, "The type of matrix does not match!");
					printUsage();
				}
				return false;
			}
						
		} else if (strcmp(arg, "-mat") == 0) {
			sscanf(argv[++index], "%s", MatrixFilePath);
			MatrixPathAvail = true;
		}else if (strcmp(arg, "-num") == 0){
			sscanf(argv[++index],"%d", &number);
			if ( number < 1 || number > 10000) {
				if (pRank == 0){
					printf("\nThe number of shuffles ('-num') should be a 1 ~ 10000.\n");
					printf("\tusing default shuffles: %d ",DEFAULT_SHUFFLE);
				}				
				shuffles_ML = DEFAULT_SHUFFLE;
			}
			shuffles_ML = number;
		}else if (strcmp(arg, "-thr") == 0){
			sscanf(argv[++index],"%d", &number);
			if ( number < 1) {
				if (pRank == 0){
					printf("\nThe number of threads is wrong.\n");
					printf("\tusing default number of threads: %d ",DEFAULT_THREADS);
				}				
				shuffles_ML = DEFAULT_THREADS;
			}			
			numThreads = number;
		}else if (strcmp(arg, "-ptype") == 0){
			sscanf(argv[++index],"%d", &number);
			if(number == 0 || number == 1){
				para_type = number;
			} else {
				if (pRank == 0){
					printf("\nThe type of parallelism is wrong (should be 0 or 1).\n");
					printf("\tusing default type: inter-parallelism");
				}				
				para_type = 0;
			}				
        }else if (strcmp(arg, "-help") == 0 || strcmp(arg, "-?") == 0) {
			printUsage();
			return false;
        }else {
		if (pRank == 0){
			fprintf(stderr,"\n************************************\n");
			fprintf(stderr,"Unknown option: %s;\n", arg);
			fprintf(stderr,"\n************************************\n");
			printUsage();
		}
		return false;
        }
	}
	
	if(DBAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the database (subject sequences) file\n");
			printUsage();
		}
		return false;
	}
	if(QueryAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the query filename File\n");
			printUsage();
		}
		return false;
	}
	if(MatrixPathAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the file path of matrix \n");
			printUsage();
			return false;
		}
	}
	if(ClsAvail == false){
		if (pRank == 0){
			fprintf(stderr, "Please specifiy the classification file \n");
			printUsage();
		}
		return false;
	}
	
	return true;
}

void CInputParse::iniStr(char* s, int len)
{
	for(int i = 0; i < len; i++)
	{ 
		s[i]='\0';
	}
}

void CInputParse::getSysTime(double *dtime)
{
  struct timeval tv;

  gettimeofday (&tv, NULL);

  *dtime = (double) tv.tv_sec;
  *dtime = *dtime + (double) (tv.tv_usec) / 1000000.0;
}

	
	